<?php

namespace Boomee\Deal\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Boomee\Deal\Http\Requests\DealRequest;
use Boomee\Deal\Interfaces\DealRepositoryInterface;
use Boomee\Deal\Models\Deal;

/**
 * Resource controller class for deal.
 */
class DealResourceController extends BaseController
{

    /**
     * Initialize deal resource controller.
     *
     * @param type DealRepositoryInterface $deal
     *
     * @return null
     */
    public function __construct(DealRepositoryInterface $deal)
    {
        parent::__construct();
        $this->repository = $deal;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Boomee\Deal\Repositories\Criteria\DealResourceCriteria::class);
    }

    /**
     * Display a list of deal.
     *
     * @return Response
     */
    public function index(DealRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Boomee\Deal\Repositories\Presenter\DealPresenter::class)
                ->$function();
        }

        $deals = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('deal::deal.names'))
            ->view('deal::deal.index', true)
            ->data(compact('deals', 'view'))
            ->output();
    }

    /**
     * Display deal.
     *
     * @param Request $request
     * @param Model   $deal
     *
     * @return Response
     */
    public function show(DealRequest $request, Deal $deal)
    {

        if ($deal->exists) {
            $view = 'deal::deal.show';
        } else {
            $view = 'deal::deal.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('deal::deal.name'))
            ->data(compact('deal'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new deal.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(DealRequest $request)
    {

        $deal = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('deal::deal.name')) 
            ->view('deal::deal.create', true) 
            ->data(compact('deal'))
            ->output();
    }

    /**
     * Create new deal.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(DealRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $deal                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('deal::deal.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('deal/deal/' . $deal->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/deal/deal'))
                ->redirect();
        }

    }

    /**
     * Show deal for editing.
     *
     * @param Request $request
     * @param Model   $deal
     *
     * @return Response
     */
    public function edit(DealRequest $request, Deal $deal)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('deal::deal.name'))
            ->view('deal::deal.edit', true)
            ->data(compact('deal'))
            ->output();
    }

    /**
     * Update the deal.
     *
     * @param Request $request
     * @param Model   $deal
     *
     * @return Response
     */
    public function update(DealRequest $request, Deal $deal)
    {
        try {
            $attributes = $request->all();

            $deal->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('deal::deal.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('deal/deal/' . $deal->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('deal/deal/' . $deal->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the deal.
     *
     * @param Model   $deal
     *
     * @return Response
     */
    public function destroy(DealRequest $request, Deal $deal)
    {
        try {

            $deal->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('deal::deal.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('deal/deal/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('deal/deal/' . $deal->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple deal.
     *
     * @param Model   $deal
     *
     * @return Response
     */
    public function delete(DealRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('deal::deal.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('deal/deal'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/deal/deal'))
                ->redirect();
        }

    }

    /**
     * Restore deleted deals.
     *
     * @param Model   $deal
     *
     * @return Response
     */
    public function restore(DealRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('deal::deal.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/deal/deal'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/deal/deal/'))
                ->redirect();
        }

    }

}
